local datafile = require "datafile"
local dir = require "pl.dir"
local etlua = require "etlua"
local pp = require "pl.pretty"

local term = require "lsup.term"
local triple = require "lsup.triple"
local graph = require "lsup.graph"

local pkar = require "pocket_archive"
local model = require "pocket_archive.model"


-- Compile all templates once.
-- TODO Add override for user-maintained templates.
local fh, idx_tpl, res_tpl, err
fh = datafile.open("templates/index.html")
idx_tpl, err = etlua.compile(fh:read("a"))
if not idx_tpl then error(err) end
fh:close()

fh = datafile.open("templates/res.html")
res_tpl, err = etlua.compile(fh:read("a"))
if not res_tpl then error(err) end
fh:close()


local M = {
    res_dir = pkar.config.htmlgen.out_dir .. "/res",
}

M.generate_resource = function(gr_uri)
    local gr = graph.get(gr_uri, pkar.store)
    local dmd = {}
    local rel = {}
    local children = {}

    local res_type
    local type_attr = gr:attr(
            gr_uri, term.new_iriref("pas:contentType", pkar.nsm))
    res_type = type_attr[1].data
    local mconf = model.models[res_type]

    local attrs = gr:connections(gr_uri, term.LINK_OUTBOUND)
    for p, ots in pairs(attrs) do
        if ((mconf.attributes or {})[p.data] or {}).type == "rel" then
            -- Relationship.
            rel[p.data] = {}
            for o in pairs(ots) do table.insert(dmd[p.data], o.data) end
        elseif p.data == "pas:firstChild" then
            -- Children.
            for o in pairs(ots) do table.insert(children, o.data) end
        elseif p.data == "pas:next" then
            -- Sibling.
            for o in pairs(ots) do ls_next = o.data break end
        else
            -- Descriptive metadata.
            dmd[p.data] = {}
            -- TODO differentiate term types
            for o in pairs(ots) do table.insert(dmd[p.data], o.data) end
        end
    end
    table.sort(dmd)
    table.sort(rel)
    table.sort(children)
    print("DMD:")
    pp.dump(dmd)
    print("REL:")
    pp.dump(rel)

    out_html = res_tpl({
        title = pkar.config.site.title or "Pocket Archive",
        uri = gr_uri,
        pkar = pkar,
        dmd = dmd,
        rel = rel,
        children = children,
        ls_next = ls_next,
    })

    local res_id = gr_uri.data:gsub(pkar.nsm:get_ns("par"), "")
    local ofh = assert(io.open(string.format(
            "%s/%s.html", M.res_dir, res_id), "w"))
    return ofh:write(out_html)
end


M.generate_resources = function()
    -- TODO parallelize
    dir.makepath(M.res_dir)
    local index_ts = graph.list(pkar.store)
    for gr_uri in pairs(index_ts) do M.generate_resource(gr_uri) end
end


M.generate_idx = function()
    local obj_idx = {}
    local index_ts = graph.list(pkar.store)
    for gr_uri in pairs(index_ts) do
        local obj = {}
        local cur
        gr = graph.get(gr_uri, pkar.store)
        -- Get all subject of type: Artifact.
        s_ts = gr:term_set(
            term.new_iriref("rdf:type", pkar.nsm), triple.POS_P,
            term.new_iriref("pas:Artifact", pkar.nsm), triple.POS_O
        )
        for s in pairs(s_ts) do
            for trp in gr:lookup(s, term.new_iriref("dc:title", pkar.nsm)) do
                obj.title = trp.o
                break  -- Only one value to use
            end
        end
        if obj.title then obj_idx[gr_uri.data] = obj end
    end

    pp.dump(obj_idx)
    out_html = idx_tpl({
        title = pkar.config.site.title or "Pocket Archive",
        pkar = pkar,
        obj_idx = obj_idx,
    })

    local ofh = assert(io.open(
            pkar.config.htmlgen.out_dir .. "/index.html", "w"))

    return ofh:write(out_html)
end


return M
